﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
    
namespace MyCompany.StudName
{
    // Панель размещения элементов управления текстом
    class MatrixPanel : TableLayoutPanel
    {
        // Заготовили массив ссылок для будущей адресации
        NumericScan[] numscan = new NumericScan[6];
    
        // Конструктор для наполнения таблицы размещения
        public MatrixPanel()
        {
            // Параметры таблицы
            this.AutoSize = true;   // Подстраиваться под содержимое
            this.Padding = new Padding(this.Font.Height);   // Внутренний отступ
            // дочерних элементов от ячеек таблицы
            this.ColumnCount = 2;   // Количество столбцов таблицы размещения
    
            this.SuspendLayout();   // Приостановить механизм размещения элементов таблицы
            // до завершения создания содержимого ячеек 
    
            // Заполнить ячейки элементами интерфейса слева-направо сверху вниз
            for (int i = 0; i < 6; i++)
            {
                // Создаем и настраиваем текстовые метки в левом столбце таблицы
                Label lbl = new Label();    // Создаем неадресуемый экземпляр
                lbl.Parent = this;      // Отдали таблице
                lbl.AutoSize = true;    // Подстраиваться под содержимое
                lbl.Anchor = AnchorStyles.Left; // Привязываем к ячейке слева
                // Нерациональный способ многократно создавать 
                // массив, а использовать только один элемент 
                lbl.Text = new string[]{
                    "X Scale:", "Y Shear:", "X Shear:",
                    "Y Scale:", "X Translate:", "Y Translate:"}[i];
    
                // Создаем и настраиваем экземпляры компонента NumericScan
                numscan[i] = new NumericScan(); // Создаем адресуемый экземпляр
                numscan[i].Parent = this;       // Отдали таблице
                numscan[i].AutoSize = true;     // Подстраиваться под содержимое
                numscan[i].Anchor = AnchorStyles.Right;// Привязываем к ячейке справа
                numscan[i].Minimum = -1000; // Нижняя граница диапазона
                numscan[i].Maximum = 1000;  // Верхняя граница диапазона
                numscan[i].DecimalPlaces = 2;   // Число значащих цифр после запятой
                // Регистрируем общий обработчик на изменения состояния объекта
                // НАБИРАТЬ ВРУЧНУЮ !!!!!!!
                numscan[i].ValueChanged += new EventHandler(MatrixPanel_ValueChanged);
            }
    
            this.ResumeLayout();   // Включить механизм размещения элементов таблицы
    
            // Заполнение интерфейсных элементов единичной 
            // матрицей преобразования через свойство класса
            this.Matrix = new System.Drawing.Drawing2D.Matrix();
        }
    
        void MatrixPanel_ValueChanged(object sender, EventArgs e)
        {
            OnChanged(EventArgs.Empty);
        }
    
        // Свойство для взаимодействия экземпляров NumericScan и объекта Matrix 
        public System.Drawing.Drawing2D.Matrix Matrix
        {
            set
            {
                // Извлекаем элементы матрицы преобразования в интерфейсные элементы
                for (int i = 0; i < 6; i++)
                    numscan[i].Value = (decimal)value.Elements[i];
            }
            get
            {
                // Создаем матрицу преобразования по значениям интерфейсных элементов
                return new System.Drawing.Drawing2D.Matrix(
                    (float)numscan[0].Value,    // X Scale:
                    (float)numscan[1].Value,    // Y Shear:
                    (float)numscan[2].Value,    // X Shear:
                    (float)numscan[3].Value,    // Y Scale:
                    (float)numscan[4].Value,    // X Translate:
                    (float)numscan[5].Value);   // Y Translate:
            }
        }

        // Объявляем событие изменения состояния интерфейсных элементов
        // по типовому библиотечному делегату EventHandler
        public event EventHandler Changed;
    
        // Метод диспетчеризации события Changed 
        protected virtual void OnChanged(EventArgs args)
        {
            // Проверяем наличие подписанных обработчиков и генерируем событие
            if (Changed != null)
                Changed(this, EventArgs.Empty);
        }
    }
}
    
namespace MyCompany.StudName
{
    // Панель отображения текста
    class DisplayPanel : Panel
    {
        // Конструктор класса
        public DisplayPanel()
        {
            // Перерисовываться самостоятельно при изменении размеров
            this.ResizeRedraw = true;
        }
    
        // Создание экземпляра матрицы преобразования текста 
        // и инициализация ее единичной матрицей 
        System.Drawing.Drawing2D.Matrix matrix =
            new System.Drawing.Drawing2D.Matrix();  // Единичная матрица
    
        // Переопределение функции прорисовки панели
        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics gr = e.Graphics;   // Извлекаем контекст устройства
            Brush brush = new SolidBrush(this.ForeColor);
    
            // Защищенно отображаем трансформированный текст
            try
            {
                gr.Transform = matrix;
                gr.DrawString(this.Text, this.Font, brush, Point.Empty);
            }
            catch (Exception ext)
            {
                gr.Transform = new Matrix();  // Сбрасываем искажения
                StringFormat strfrm = new StringFormat();
                strfrm.Alignment = strfrm.LineAlignment = StringAlignment.Center;
                gr.DrawString(ext.Message, Font, brush, ClientRectangle, strfrm);
            }
    
            //Освободили ограниченный ресурс  
            //gr.Dispose();
            brush.Dispose();
    
            base.OnPaint(e);    // Спустили предкам
        }

        // Свойство доступа к матрице преобразования
        public System.Drawing.Drawing2D.Matrix Matrix
        {
            set
            {
                matrix = value;
                this.Invalidate();  // Перерисовать
            }
            // Аксессор get нам не нужен
        }
    }
}
